"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.fireInputEvent = fireInputEvent;

var _dom = require("@testing-library/dom");

var _document = require("../../document");

var _isElementType = require("../misc/isElementType");

var _selection = require("../focus/selection");

function fireInputEvent(element, {
  newValue,
  newSelection,
  eventOverrides
}) {
  const oldValue = element.value; // apply the changes before firing the input event, so that input handlers can access the altered dom and selection

  if ((0, _isElementType.isElementType)(element, ['input', 'textarea'])) {
    (0, _document.setUIValue)(element, newValue);
  } else {
    // The pre-commit hooks keeps changing this
    // See https://github.com/kentcdodds/kcd-scripts/issues/218

    /* istanbul ignore else */
    // eslint-disable-next-line no-lonely-if
    if (newSelection.node.nodeType === 3) {
      newSelection.node.textContent = newValue;
    } else {
      // TODO: properly type guard
      throw new Error('Invalid Element');
    }
  }

  (0, _selection.setSelection)({
    focusNode: newSelection.node,
    anchorOffset: newSelection.offset,
    focusOffset: newSelection.offset
  }); // When the input event happens in the browser, React executes all event handlers
  // and if they change state of a controlled value, nothing happens.
  // But when we trigger the event handlers in test environment,
  // the changes are rolled back by React before the state update is applied.
  // Then the updated state is applied which results in a resetted cursor.
  // There is probably a better way to work around  if we figure out
  // why the batched update is executed differently in our test environment.

  (0, _document.startTrackValue)(element);

  _dom.fireEvent.input(element, { ...eventOverrides
  });

  const tracked = (0, _document.endTrackValue)(element);

  if ((tracked == null ? void 0 : tracked.length) === 2 && tracked[0] === oldValue && tracked[1] === newValue) {
    (0, _selection.setSelection)({
      focusNode: newSelection.node,
      anchorOffset: newSelection.offset,
      focusOffset: newSelection.offset
    });
  }
}