"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.prepareInput = prepareInput;

var _utils = require("../../utils");

function prepareInput(data, element, inputType = 'insertText') {
  const inputRange = (0, _utils.getInputRange)(element); // TODO: implement for ranges on multiple nodes

  /* istanbul ignore if */

  if (!inputRange || 'startContainer' in inputRange && inputRange.startContainer !== inputRange.endContainer) {
    return;
  }

  const node = getNode(element, inputRange);
  const {
    newValue,
    newOffset,
    oldValue
  } = (0, _utils.calculateNewValue)(data, node, inputRange, inputType);

  if (newValue === oldValue && newOffset === inputRange.startOffset && newOffset === inputRange.endOffset) {
    return;
  }

  return {
    newValue,
    commit: () => (0, _utils.fireInputEvent)(element, {
      newValue,
      newSelection: {
        node,
        offset: newOffset
      },
      eventOverrides: {
        inputType
      }
    })
  };
}

function getNode(element, inputRange) {
  if ('startContainer' in inputRange) {
    if (inputRange.startContainer.nodeType === 3) {
      return inputRange.startContainer;
    }

    try {
      return inputRange.startContainer.insertBefore(element.ownerDocument.createTextNode(''), inputRange.startContainer.childNodes.item(inputRange.startOffset));
    } catch {
      /* istanbul ignore next */
      throw new Error('Invalid operation. Can not insert text at this position. The behavior is not implemented yet.');
    }
  }

  return element;
}